// author:   Samuel Mueller 
// version:  0.0.9 
// license:  MIT 
// homepage: http://github.com/ssmm/angular-smart-search 
(function() {
  var padding;

  padding = 30;

  angular.module("angular-smart-search", []);

  angular.module("angular-smart-search").directive("smartSearch", [
    "$http", "searchQueryFactory", "templateFactory", function($http, searchQueryFactory, templateFactory) {
      var QueryState;

      QueryState = function() {
        this.state = 'initial';
        this.query = void 0;
        this.is = function(state) {
          return this.state === state;
        };
        this.setTo = function(state) {
          return this.state = state;
        };
      };
      return {
        restrict: "A",
        scope: {
          stub: "="
        },
        compile: function(element, attributes, transclude) {
          var detailOverlay, id_field, template;

          if (attributes.detailOverlay) {
            detailOverlay = jQuery("#" + attributes.detailOverlay);
            if (!detailOverlay[0]) {
              detailOverlay = void 0;
            }
          }
          template = angular.element(templateFactory.getTemplate(angular.isDefined(detailOverlay)));
          if (detailOverlay) {
            angular.element(template.find("detail-overlay")).append(detailOverlay.html());
          }
          id_field = angular.element("<input ng-model='result.id' type='text' style='width: 30px; display: none;'>");
          id_field.attr("id", attributes.id);
          id_field.attr("name", attributes.name);
          template.append(id_field);
          element.replaceWith(template);
          return {
            post: function($scope, $element, $attributes) {
              var allowedFormats, detailOverlayTrigger, hideOverlay, outerQueryTimeout, queryState, quickQueryInput, searchInput, searchOverlay, service, setResult, showOverlay, successCallbacks;

              successCallbacks = [];
              queryState = new QueryState();
              $scope.queryState = queryState;
              $scope.resultList = [];
              allowedFormats = $attributes.formats.split(" ");
              service = $attributes.service;
              if ($attributes.init) {
                $scope.result = angular.fromJson($attributes.init)[0];
                $scope.queryState.setTo("successful");
              }
              quickQueryInput = $element.find("[quick-query-input]");
              searchInput = $element.find("[search-input]");
              detailOverlayTrigger = $element.find("[detail-overlay-trigger]");
              searchOverlay = $element.children("search-overlay");
              detailOverlay = $element.children("detail-overlay");
              showOverlay = function(overlay, origin, action) {
                overlay.css("top", origin.top - padding);
                overlay.css("left", origin.left - padding);
                return overlay.show("fade", action);
              };
              $scope.showSearchOverlay = function() {
                return showOverlay(searchOverlay, quickQueryInput.position(), function() {
                  return searchInput.focus();
                });
              };
              $scope.showDetailOverlay = function() {
                $scope.item = $scope.result;
                return showOverlay(detailOverlay, detailOverlayTrigger.position());
              };
              hideOverlay = function(overlay, action) {
                return overlay.hide("fade", action);
              };
              $scope.hideSearchOverlay = function() {
                return hideOverlay(searchOverlay, function() {
                  return quickQueryInput.focus();
                });
              };
              $scope.hideDetailOverlay = function() {
                return hideOverlay(detailOverlay);
              };
              $scope.search = function(query) {
                return $http.get("" + service + ".json?fulltext=" + query).success(function(data) {
                  $scope.quickQuery = "";
                  return $scope.resultList = data;
                }).error(function() {});
              };
              $scope.pick = function(item) {
                setResult(item);
                return $scope.hideSearchOverlay();
              };
              $scope.hover = function(item) {
                return $scope.currentItem = item;
              };
              outerQueryTimeout = null;
              $scope.outerQueryChanged = function(query) {
                window.clearTimeout(outerQueryTimeout);
                if (!query) {
                  if ($scope.result) {
                    return queryState.setTo("successful");
                  }
                } else {
                  query = searchQueryFactory(query, allowedFormats);
                  if (query) {
                    queryState.setTo("changed");
                    queryState.query = query;
                    return outerQueryTimeout = window.setTimeout(function() {
                      queryState.setTo("pending");
                      $http.get("" + service + ".json?" + query.format.name + "=" + query.query).success(function(data) {
                        if (data[0] && data[0].id) {
                          return setResult(data[0]);
                        } else {
                          return queryState.setTo("unsuccessful");
                        }
                      }).error(function() {
                        return queryState.setTo("error");
                      });
                      return $scope.$apply();
                    }, 1200);
                  } else {
                    return queryState.setTo("invalid-format");
                  }
                }
              };
              setResult = function(result) {
                var callback, _i, _len, _results;

                queryState.setTo("successful");
                $scope.result = result;
                if (successCallbacks.length > 0) {
                  _results = [];
                  for (_i = 0, _len = successCallbacks.length; _i < _len; _i++) {
                    callback = successCallbacks[_i];
                    _results.push(callback.call(void 0, result));
                  }
                  return _results;
                }
              };
              if ($attributes.stub) {
                return $scope.stub = {
                  registerSuccessCallback: function(callback) {
                    return successCallbacks.push(callback);
                  },
                  reset: function() {
                    $scope.result = void 0;
                    $scope.quickQuery = "";
                    return queryState.setTo("");
                  },
                  set: function(value) {
                    return setResult(value);
                  }
                };
              }
            }
          };
        }
      };
    }
  ]);

  angular.module("angular-smart-search").provider("markupStore", [
    function() {
      var infoIconClass, overlayClass, searchIconClass, successIconClass;

      successIconClass = "icon-info-sign icon-large";
      infoIconClass = "icon-info-sign";
      searchIconClass = "icon-search";
      overlayClass = "smart-search-overlay";
      this.setSuccessIconClass = function(attr) {
        return successIconClass = attr;
      };
      this.setInfoIconClass = function(attr) {
        return infoIconClass = attr;
      };
      this.setSearchIconClass = function(attr) {
        return searchIconClass = attr;
      };
      this.setOverlayClass = function(attr) {
        return overlayClass = attr;
      };
      return this.$get = function() {
        return {
          successIconClass: function() {
            return successIconClass;
          },
          infoIconClass: function() {
            return infoIconClass;
          },
          searchIconClass: function() {
            return searchIconClass;
          },
          overlayClass: function() {
            return overlayClass;
          }
        };
      };
    }
  ]);

  angular.module("angular-smart-search").provider("searchQueryFactory", [
    function() {
      var Query, detectFormat, formats;

      Query = function(query, format) {
        this.format = format;
        this.query = query;
      };
      formats = {};
      detectFormat = function(query, allowedFormats) {
        var name, _i, _len;

        for (_i = 0, _len = allowedFormats.length; _i < _len; _i++) {
          name = allowedFormats[_i];
          if (formats[name] && formats[name].pattern.exec(query)) {
            return formats[name];
          }
        }
      };
      this.registerFormats = function(formats) {
        var format, _i, _len, _results;

        _results = [];
        for (_i = 0, _len = formats.length; _i < _len; _i++) {
          format = formats[_i];
          _results.push(this.registerFormat(format));
        }
        return _results;
      };
      this.registerFormat = function(format) {
        return formats[format.name] = format;
      };
      return this.$get = function() {
        return function(query, allowedFormats) {
          var format;

          format = detectFormat(query, allowedFormats);
          if (format) {
            return new Query(query, format);
          }
        };
      };
    }
  ]);

  angular.module("angular-smart-search").service("templateFactory", [
    "markupStore", function(markupStore) {
      var detailOverlay, detailOverlayTrigger, inputSection, overlayStyle, resultIcon, searchOverlay, showDetailOverlayAction, successIcon;

      overlayStyle = "style='    display: none; position: absolute;    background-color: white; padding: 30px;    border: solid;    border-width: 1px;    border-color: rgb(221, 221, 221);    border-radius: 10px;  '";
      detailOverlayTrigger = "<i detail-overlay-trigger class='" + (markupStore.infoIconClass()) + "' ng-click='showDetailOverlay()'></i>";
      successIcon = "<i class='" + (markupStore.successIconClass()) + "'></i>";
      showDetailOverlayAction = function(withDetailOverlay) {
        if (withDetailOverlay) {
          return "ng-click='showDetailOverlay()'";
        }
      };
      resultIcon = function(withDetailOverlay) {
        if (withDetailOverlay) {
          return detailOverlayTrigger;
        } else {
          return successIcon;
        }
      };
      inputSection = function(withDetailOverlay) {
        return "      <div>        <!-- input section you see initially -->        <div class='input-prepend input-append'>          <!-- gray display section -->          <span class='add-on' style='min-width: 200px;'>            <!-- result if found -->            <span ng-show='queryState.is(\"successful\")' " + (showDetailOverlayAction(withDetailOverlay)) + ">              " + (resultIcon(withDetailOverlay)) + "              {{result.displayText}}            </span>            <!-- query states -->            <span ng-show='queryState.is(\"changed\")'>              <em>{{queryState.query.format.name}} detected</em>            </span>            <span ng-show='queryState.is(\"pending\")'>              <em>searching...</em>            </span>            <span ng-show='queryState.is(\"unsuccessful\")'>              <em>Not found</em>            </span>            <span ng-show='queryState.is(\"invalid-format\")'>              <em>Invalid format</em>            </span>            <span ng-show='queryState.is(\"error\")'>              <em>Error</em>            </span>          </span>          <!-- quick query field -->          <input ng-change='outerQueryChanged(quickQuery)' ng-model='quickQuery' quick-query-input style='width: 100px;' type='text'>          <!-- button that triggers detail search overlay -->          <button type='button' class='btn' ng-click='showSearchOverlay()'>            <i class='" + (markupStore.searchIconClass()) + "'></i>          </button>        </div>        " + (detailOverlay()) + "        " + (searchOverlay()) + "      </div>    ";
      };
      searchOverlay = function() {
        return "      <search-overlay " + overlayStyle + " class='" + (markupStore.overlayClass()) + "' ng-mouseleave='hideSearchOverlay()'>        <div class='row-fluid'>          <div class='span12'>            <div class='input-append'>              <form>                <input search-input ng-model='detailQuery' type='text'>                <button type='submit' class='btn' ng-click='search(detailQuery)'>                  <i class='" + (markupStore.searchIconClass()) + "'></i>                </button>              </form>            </div>          </div>        </div>        <div class='horizontal-space'></div>        <div class='row-fluid'>          <div class='span12'>            <table at-table class='table table-bordered table-hover' fill-last-page pagination='wildPagination'>              <thead></thead>              <tbody>                <tr ng-mouseover='hover(item)'>                  <td at-implicit attribute='id' class='sortable 50px'></td>                  <td at-implicit attribute='displayText' class='sortable 280px' title='Result'></td>                  <td class='50px' title=''>                    <button type='button' class='btn btn-mini' ng-click='pick(item)'>pick</button>                  </td>                </tr>              </tbody>            </table>            <at-pagination instance='wildPagination' items-per-page='3' list='resultList'></at-pagination>          </div>        </div>      </search-overlay>    ";
      };
      detailOverlay = function() {
        return "      <detail-overlay " + overlayStyle + " class='" + (markupStore.overlayClass()) + "' ng-mouseleave='hideDetailOverlay()'>      </detail-overlay>    ";
      };
      return {
        getTemplate: function(withDetailOverlay) {
          return inputSection(withDetailOverlay);
        }
      };
    }
  ]);

}).call(this);
